// Adapted from
//   http://www.hackerscapes.com/2014/11/how-to-save-data-from-arduino-to-a-csv-file-using-processing/
//   by Dr. YIP Chi Lap [Beta]
//   for Ultraviolet Radiation Measurement and Application Design Competition

import processing.serial.*;

// Creates a software serial port on which you will listen to Arduino
Serial myPort;
// Table where we will read in and store values.
// You can name it something more creative!
Table dataTable;
String fileName;

int numReadings = 10;   // Number of readings to take before writing the file.
int readingCounter = 0; // Counts each reading to compare to numReadings.
int portNumber = 5;     // This may be different in your environment.
                        // See the list of port names when the program is run
                        //   and change this variable.  Also see comments below.


void setup() {
  dataTable = new Table();
  // Print the list of serial ports so the port number can be set to the right one.
  String[] portNames = Serial.list();
  for (int r=0; r<portNames.length; ++r) {
    print(r);
    print(" ");
    println(portNames[r]);
  }
  String portName = portNames[portNumber];
  println("---------------");
  print("Serial port used: ");
  println(portName);
  // CAUTION: your Arduino port number is probably different!
  // Use a "handshake" sketch to figure out and
  //   test which port number your Arduino is talking on.
  // A "handshake" establishes that Arduino and Processing are
  //   listening/talking on the same port.
  // Here's a link to a basic handshake tutorial:
  //   https://processing.org/tutorials/overview/

  // Set up your port to listen to the serial port
  myPort = new Serial(this, portName, 9600);

  // This column stores a unique identifier for each record.
  // We will just count up from 0 -
  //   so your first reading will be ID 0, your second will be ID 1, etc.
  dataTable.addColumn("id");

  // the following are dummy columns for each data value.
  // Add as many columns as you have data values.
  // Customize the names as needed.
  // Make sure they are in the same order as the order that
  //   Arduino is sending them!
  dataTable.addColumn("UVIndex");
  dataTable.addColumn("Voltage");

  // The following adds columns for time.
  // You can also add milliseconds.
  // See the Time/Date functions for Processing:
  //   https://www.processing.org/reference/
  dataTable.addColumn("year");
  dataTable.addColumn("month");
  dataTable.addColumn("day");
  dataTable.addColumn("hour");
  dataTable.addColumn("minute");
  dataTable.addColumn("second");
}



void serialEvent(Serial myPort) {
  // The newline separator separates each Arduino loop.
  // We will parse the data by each newline separator.
  String val = myPort.readStringUntil('\n');
  if (val!= null) { // We have a reading! Record it.
    // Gets rid of any whitespace or Unicode nonbreakable space
    val = trim(val);
    // Optional, useful for debugging.
    // If you see this, you know data is being sent.
    // Delete if you like.
    println(val);
    // Parses the packet from Arduino and
    //   places the valeus into the sensorVals array.
    // I am assuming floats.
    // Change the data type to match the datatype coming from Arduino.
    float sensorVals[] = float(split(val, ' '));

    // Add a row for this new reading
    TableRow newRow = dataTable.addRow();
    // Record a unique identifier (the row's index)
    newRow.setInt("id", dataTable.lastRowIndex());

    // Record sensor information.
    // Customize the names so they match your sensor column names.
    newRow.setFloat("UVIndex", sensorVals[0]);
    newRow.setFloat("Voltage", sensorVals[1]);

    // Record time stamp
    newRow.setInt("year", year());
    newRow.setInt("month", month());
    newRow.setInt("day", day());
    newRow.setInt("hour", hour());
    newRow.setInt("minute", minute());
    newRow.setInt("second", second());

    // Optional, use if you'd like to
    //   write your file every numReadings reading cycles.
    readingCounter++;

    // Saves the table as a csv
    //   in the same folder as the sketch every numReadings.
    if (readingCounter % numReadings ==0) {
      // The % is a modulus,
      //   a math operator that signifies remainder after division.
      // The if statement checks if readingCounter is a multiple of numReadings
      //   (the remainder of readingCounter/numReadings is 0)
      fileName = str(year()) + "-" + str(month()) + "-" + str(day()) + ".csv";
      // Save the data to your computer.
      // It is ready for all your spreadsheet dreams.
      saveTable(dataTable, fileName);
    }
  }
}



void draw() {
  // Consider visualising using Processing as well,
  //   and put visualisation code here.
}
